import {IBackendConnector} from "./IBackendConnector.ts";
import {Plant, PlantSchema, searchRequest} from "./schemas.ts";
import {ZodError} from "zod";
import {FetchResult} from "./FetchResult.ts";

export class BackendConnectorImpl implements IBackendConnector {
    baseUrl: string;

    constructor(baseUrl: string) {
        this.baseUrl = baseUrl;
    }

    async getAllPlants(page?: number, pageSize?: number): Promise<FetchResult<Plant[]>> {
        let result: FetchResult<Plant[]> = {};
        try {
            const response = await fetch(this.baseUrl + "/plants?page=" + page + "&pageSize=" + pageSize);
            const json = await response.json();
            result = this.parsePlants(json);
        } catch (e) {
            console.error("An error occurred while fetching all plants")
            console.error(e);
            result = {...result, err: e};
        }
        return result;
    }

    async searchPlants(search: searchRequest): Promise<FetchResult<Plant[]>> {
        let result: FetchResult<Plant[]> = {};
        try {
            const params = new URLSearchParams(
                Object.entries(search)
                    .map(([key, value]) => [key, value as string])
            );
            const response = await fetch(this.baseUrl + "/search" + params);
            const json = await response.json();
            result = this.parsePlants(json);
        } catch (e) {
            console.error("An error occurred while searching through plants")
            console.error(e);
            result = {...result, err: e};
        }
        return result;
    }

    async getRandomPlants(amount: number): Promise<FetchResult<Plant[]>> {
        let result: FetchResult<Plant[]> = {};
        try {
            const response = await fetch(this.baseUrl + "/randomPlants/" + amount);
            const json = await response.json();
            result = this.parsePlants(json);
        } catch (e) {
            console.error("An error occurred while getting random plants")
            console.error(e);
            result = {...result, err: e};
        }
        return result;
    }

    async getSinglePlant(id: number): Promise<FetchResult<Plant | null>> {
        let result: FetchResult<Plant> = {};
        try {
            const response = await fetch(this.baseUrl + "/" + id);
            const json = await response.json();
            result = this.parseSinglePlant(json);
        } catch (e) {
            console.error("An error occurred while fetching plant with ID", id);
            console.error(e);
            result = {...result, err: e};
        }
        return result;
    }

    async addToGarden(plantId: number): Promise<FetchResult<void>> {
        let result: FetchResult<void> = {};
        try {
            await fetch(this.baseUrl + "/garden/add" + plantId);
        } catch (e) {
            console.error("An error occurred while adding plant with ID", plantId, "to garden");
            console.error(e);
            result = {err: e};
        }
        return result;
    }

    async clearGarden(): Promise<FetchResult<void>> {
        let result: FetchResult<void> = {};
        try {
            await fetch(this.baseUrl + "/garden/remove/all");
        } catch (e) {
            console.error("An error occurred while clearing garden");
            console.error(e);
            result = {err: e};
        }
        return result;
    }

    async getGardenEntries(sort?: string): Promise<FetchResult<Plant[]>> {
        let result: FetchResult<Plant[]> = {};
        try {
            const params = new URLSearchParams();
            if (sort) params.set("sort", sort);
            const response = await fetch(this.baseUrl + "/garden" + params);
            const json = await response.json();
            result = this.parsePlants(json);
        } catch (e) {
            console.error("An error occurred while getting garden entries")
            console.error(e);
            result = {...result, err: e};
        }
        return result;
    }

    async getGardenSize(): Promise<FetchResult<number>> {
        let result: FetchResult<number> = {};
        try {
            const response = await fetch(this.baseUrl + "/garden/count");
            const json = await response.json();
            const size = Number.parseInt(json);
            result = {value: size};
        } catch (e) {
            console.error("An error occurred while fetching garden size")
            console.error(e);
            result = {...result, err: e};
        }
        return result;
    }

    async removeEntryFromGarden(entryId: number): Promise<FetchResult<void>> {
        let result: FetchResult<void> = {};
        try {
            await fetch(this.baseUrl + "/garden/remove/" + entryId);
        } catch (e) {
            console.error("An error occurred while removing garden entry with ID", entryId);
            console.error(e);
            result = {err: e};
        }
        return result;
    }

    parseSinglePlant(json: any): FetchResult<Plant> {
        let result: FetchResult<Plant> = {};
        try {
            result = {value: PlantSchema.parse(json)};
        } catch (e) {
            if (e instanceof ZodError) {
                result = {...result, err: "One or more plants could not be parsed correctly."}
            }
        }
        return result;
    }

    parsePlants(json: any): FetchResult<Plant[]> {
        let result: FetchResult<Plant[]> = {};
        const plants: Plant[] = [];
        for (const p of json) {
            try {
                plants.push(PlantSchema.parse(p))
            } catch (e) {
                if (e instanceof ZodError) {
                    result = {...result, err: "One or more plants could not be parsed correctly."}
                }
            }
        }
        return {...result, value: plants};
    }
}
